package sda.jdbc.intro.auth;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;

import sda.jdbc.intro.v1.DbProps;


/**
 * Handle database communication. There are two
 * distinct internal states <q>disconnected</q> and <q>connected</q>, see
 * {@link #isConnected()}. These two states may be toggled by invoking
 * {@link #connect()} and {@link #disconnect()} respectively.
 * 
 * The following snippet illustrates the intended usage:
 * <pre>   public static void main(String[] args) {
      final PersistenceHandler ph = new PersistenceHandler();
      if (ph.connect()) {
         if (!ph.add("Jim", "jim@foo.com")) {
            System.err.println("Insert Error:" + ph.getErrorMessage());
         }
      } else {
         System.err.println("Connect error:" + ph.getErrorMessage());
      }
   }</pre>
 * 
 * @author goik
 */

public class PersistenceHandler {
   /**
    * See {@link PreparedStatementHandler}
    * @author goik
    *
    */
   class PrepareUserInsert extends PreparedStatementHandler {

      public PrepareUserInsert(final Connection conn) throws SQLException {
         super(conn,
               "INSERT INTO Person (name, email, login, password) VALUES (?, ?, ?, ?)");
      }                          // 1     2      3      4                 1  2  3  4
      /**
       * This method needs to be adjusted whenever the SQL string in {@link PrepareUserInsert}
       * changes.
       * 
       * @param name
       * @param email
       * @param login
       * @param password
       * @throws SQLException
       */
      public void setParameter(
            final String name,
            final String email,
            final String login, 
            final char[] password) throws SQLException {
         
         pStmt.setString(1, name);
         pStmt.setString(2, email);
         pStmt.setString(3, login);
         pStmt.setString(4, HashProvider.getSaltedHash(password));
      }
   }

   class PrepareFindUserByLogin extends PreparedStatementHandler {

      public PrepareFindUserByLogin(final Connection conn) throws SQLException {
         super(conn, "SELECT * FROM Person WHERE login = ?");
      }
      public void setParameter(final String login) throws SQLException {
         pStmt.setString(1, login);
      }
   }

   Connection conn = null;

   PrepareUserInsert userInsert;
   PrepareFindUserByLogin findUserByLogin;
   
   String errorMessage = null;

   /**
    * New instances are in <q>disconnected</q> state. See {@link #isConnected()}
    */
   public PersistenceHandler() {/* only present here to supply javadoc comment */}

   /**
    * Inserting a (name, email) record into the database server. In case of
    * errors corresponding messages may subsequently be retrieved by calling
    * {@link #getErrorMessage()}.
    * 
    * <dt><b>Precondition:</b></dt> <dd>must be in
    * <q>connected</q> state, see {@link #isConnected()}</dd>
    * 
    * @param name
    *           A person's name
    * @param email
    *           A person's email address
    * 
    * @return true if the current data record has been successfully inserted
    *         into the database server. false in case of error(s).
    */
   public boolean add(final String name,
                        final String email,
                        final String login,
                        final char[] password){
      try {
         userInsert.setParameter(name, email, login, password);
         userInsert.executeUpdate();
         return true;
      } catch (SQLException e) {
         errorMessage = "Unable to insert '" + name + ", " + email + "': '" + e.getMessage() + "'";
         return false;
      }
   }

   /**
    * Retrieving error messages in case a call to {@link #add(String, String)},
    * {@link #connect()}, or {@link #disconnect()} yields an error.
    * 
    * @return the error explanation corresponding to the latest failed
    *         operation, null if no error yet occurred.
    */
   public String getErrorMessage() {
      return errorMessage;
   }

   /**
    * Open a connection to a database server.
    * 
    * <dt><b>Precondition:</b><dd>
    *  <dd>must be in <q>disconnected</q> state, see {@link #isConnected()}</dd>
    *  
    * <dt><b>Precondition:</b><dd>
    *  <dd>The following properties must be set:
    *  <pre>PersistenceHandler.jdbcUrl=jdbc:mysql://localhost:3306/hdm
PersistenceHandler.password=XYZ
PersistenceHandler.username=foo</pre>
    *  </dd>
    * 
    * @return true if connecting was successful
    */
   public boolean connect () {
      try {
         conn = DriverManager.getConnection(
               DbProps.getString("PersistenceHandler.jdbcUrl"),
               DbProps.getString("PersistenceHandler.username"),
               DbProps.getString("PersistenceHandler.password"));
         try {
            userInsert = new PrepareUserInsert(conn);
            findUserByLogin = new PrepareFindUserByLogin(conn);
            return true; 
         } catch (SQLException e) {
            errorMessage = "Connection opened but Statement creation failed:\"" + e.getMessage() + "\"."; 
            try {
               conn.close();
            } catch (SQLException ee) {
               errorMessage += "Closing connection failed:\"" + e.getMessage() + "\".";
            }
            conn = null;
         }

      } catch (SQLException e) {
         errorMessage = "Unable to open connection:\"" + e.getMessage() + "\".";
      }
      return false;
   }

   /**
    * Close a connection to a database server and clean up JDBC related resources
    * 
    * Error messages in case of failure may subsequently be retrieved by
    * calling {@link #getErrorMessage()}.
    * 
    * <dt><b>Precondition:</b></dt>
    * <dd>must be in <q>connected</q> state, see {@link #isConnected()}</dd>
    * 
    * @return true if disconnecting was successful, false in case error(s) occur.
    */
   public boolean disconnect() {
      boolean resultStatus = true;
      final StringBuffer messageCollector = new StringBuffer();
      try {
         conn.close();
      } catch (SQLException e) {
         resultStatus = false;
         messageCollector.append("Unable to close connection:\"" + e.getMessage() + "\".");
      }
      conn = null;
      if (!resultStatus) {
         errorMessage = messageCollector.toString();
      }
      return resultStatus;
   }

   /**
    * An instance can either be in <q>connected</q> or <q>disconnected</q> state. The
    * state can be toggled by invoking {@link #connect()} or
    * {@link #disconnect()} respectively.
    * 
    * @return true if connected, false otherwise
    */
   public boolean isConnected() {
      return null != conn;
   }
   /**
    * Check, whether a user is allowed to log in. The credentials
    * must match the attribute values (login,password) in 
    * the relation Person. 
    * 
    * <dt><b>Precondition:</b></dt>
    * <dd>must be in <q>connected</q> state, see {@link #isConnected()}</dd>
    * 
    * @param login The user's login name entered in GUI
    * @param guiPassword The user's password entered in GUI
    * @return
    */
   public boolean isAuthorized(final String login, final char[] guiPassword) {
      try {
         findUserByLogin.setParameter(login);
         final ResultSet user = findUserByLogin.executeQuery();
         if (user.next()) {
            final String dbPassword = user.getString("password");
            if (HashProvider.check(guiPassword, dbPassword)) {
               return true;
            }
            errorMessage = "invalid credentials";
         } else {
            errorMessage = "invalid credentials";
         }
      } catch (SQLException e) {
         errorMessage = e.getLocalizedMessage();
      }
      return false;
   }
}