package sda.jdbc.intro.v1.sanitize;

import java.util.regex.Pattern;

import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTextField;

/**
 * We may check user input from text fields in a generic way. 
 * defined by regular expressions. In addition error messages are forwarded
 * automatically to respective GUI elements.
 * 
 * @author goik
 */
public class RegexpVerifier extends InputVerifier {

   final Pattern syntaxPattern;
   public final JLabel validationLabel;
   private boolean inputValid = false;
   private final String errMsg; 
   
   /**
    * For each input field to be validated we need the
    * following parameters:

    * @param regex The regular expression defining allowed textual content. The string
    *                "^[^ ]*$" for example allows arbitrary text not containing any spaces.
    * @param validationLabel  This is where error messages in case of rexex violation go.
    * @param errMsg      This text will be presented to the user in case of a validation error.
    */
   public RegexpVerifier (final String regex, final String errMsg) {
      this.errMsg = errMsg;
      syntaxPattern = Pattern.compile(regex);
      validationLabel = new JLabel();
   }

   @Override
   public boolean verify(JComponent input) {
      if (input instanceof JTextField) {
         final String userInput =  ((JTextField) input).getText();
         if (syntaxPattern.matcher(userInput).find()) {
            validationLabel.setText("");
            inputValid = true;
         } else {
            validationLabel.setText(errMsg);
            inputValid = false;
         }
      }
      return inputValid;
   }
   
   /**
    * @return the current validation state.
    */
   public boolean inputIsValid () {
      return inputValid;
   }
}
