package solve.dom;

import java.io.IOException;
import java.io.PrintStream;

import org.jdom2.DocType;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.Text;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

/**
 * Holding a HTML DOM to produce output.
 * @author goik
 */
public class HtmlTree {

   private Document htmlOutput;
   private Element tableBody;

   /**
    * Create a HTML skeleton (<html><head>...<body> ...</body></head>
    * to be filled in later by calling {@link #appendEntrySet(String[])}.
    * @param domImplLs
    *   A DOM implementation with "LS" feature enabled
    * @param titleText 
    *   The document's title, e.g. "Available articles"
    * @param tableHeaderFields
    *   The articles will be displayed as HTML table with these header fields
    *   e.g. {"Article Description", "Order Number" }
    */
   public HtmlTree(final String titleText,
         final String[] tableHeaderFields) {

      DocType doctype =  new DocType("html",
            "-//W3C//DTD XHTML 1.0 Strict//EN", 
            "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd");

      final Element htmlRoot = new Element("html");
      htmlOutput = new Document(htmlRoot);
      htmlOutput.setDocType(doctype);

      // We create a HTML skeleton including an "empty" table
      final Element head = new Element("head"),
            body = new Element("body"),
            table = new Element("table");

      htmlRoot.addContent(head).addContent(body);

      head.addContent(new Element("title").addContent(new Text(titleText)));
      
      body.addContent(new Element("h1").addContent(new Text(titleText)));

      body.addContent(table);


      tableBody = new Element("tbody");
      table.addContent(tableBody);
      
      final Element tr = tableBody.addContent(new Element("tr"));
      for (final String headerField:  tableHeaderFields) {
         tr.addContent(new Element("th").addContent(new Text(headerField)));
      }
   }
   
   /**
    * Inserting an <item> as a new table row with two <td> elements.
    * 
    * @param itemName
    * @param orderNo
    */
   public void appendItem(final String itemName, final String orderNo) {
      final Element tr = new Element("tr");
      tableBody.addContent(tr);
      tr.addContent(new Element("td").addContent(new Text(itemName)));
      tr.addContent(new Element("td").addContent(new Text(orderNo)));
   }
   /**
    * Serialize the HTML DOM to a stream
    * @param out
    */
   public void serialize(PrintStream out){

      // Set formatting for the XML output
      final Format outFormat = Format.getPrettyFormat();

      // Serialize to console
      final XMLOutputter printer = new XMLOutputter(outFormat);
      try {
         printer.output(htmlOutput, System.out);
      } catch (IOException e) {
         e.printStackTrace();
         System.exit(1);
      }
   }
   /**
    * @return the table's <tbody> element
    */
   public Element getTable() {
      return tableBody;
   }
}